package com.wombat.aliyun_oss

import android.app.Activity
import android.app.Activity.RESULT_OK
import android.content.Intent
import android.net.Uri
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.annotation.NonNull
import com.alibaba.sdk.android.oss.ClientConfiguration
import com.alibaba.sdk.android.oss.OSSClient
import com.alibaba.sdk.android.oss.common.auth.OSSPlainTextAKSKCredentialProvider
import com.alibaba.sdk.android.oss.common.auth.OSSStsTokenCredentialProvider
import com.alibaba.sdk.android.oss.common.utils.BinaryUtil
import com.wombat.aliyun_oss.listener.UploadCallback
import com.wombat.aliyun_oss.model.UploadFailData
import com.wombat.aliyun_oss.model.UploadInitData
import com.wombat.aliyun_oss.model.UploadProgressData
import com.wombat.aliyun_oss.model.UploadSuccessData
import com.wombat.aliyun_oss.service.OSSService
import com.wombat.aliyun_oss.utils.MapUtil
import com.wombat.aliyun_oss.utils.RealPathUtil
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import kotlin.concurrent.thread

/** AliyunOSSPlugin */
class AliyunOSSPlugin : FlutterPlugin, MethodCallHandler, ActivityAware,
    PluginRegistry.ActivityResultListener, EventChannel.StreamHandler {
    private lateinit var methodChannel: MethodChannel
    private lateinit var eventChannel: EventChannel
    private var activity: Activity? = null
    private var ossService: OSSService? = null
    private var activityPluginBinding: ActivityPluginBinding? = null
    private var bucketName: String = ""

    private val resultSelectFile = 101

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel =
            MethodChannel(flutterPluginBinding.binaryMessenger, "com.wombat/aliyun_oss_plugin")

        methodChannel.setMethodCallHandler(this)

        eventChannel = EventChannel(
            flutterPluginBinding.binaryMessenger,
            "com.wombat/aliyun_oss_stream_plugin"
        )
        eventChannel.setStreamHandler(this)

    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
        binding.addActivityResultListener(this)
        activityPluginBinding = binding
    }

    override fun onDetachedFromActivityForConfigChanges() {
        TODO("Not yet implemented")
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        TODO("Not yet implemented")
    }

    override fun onDetachedFromActivity() {
        activity = null
        activityPluginBinding?.removeActivityResultListener(this)
        activityPluginBinding = null
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        // 用于处理选择文件的结果监听
        if (resultCode != RESULT_OK) {
            return false
        }
        if (requestCode == resultSelectFile && null != data) {
            val filePath = RealPathUtil.getRealPath(activity!!, data.data!!)
            if (filePath != null) {
                upload(bucketName, data.data!!, filePath)
                return true
            }
        }
        return false
    }


    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "init" -> {
                initOSSClient(call.arguments as HashMap<String, Any>)
                result.success("初始化成功")
            }
            "getMd5Str" -> {
                val filePath = call.argument<String>("filePath") as String
                thread {
                    val start = System.currentTimeMillis()
                    Log.d("计算md5", filePath)
                    val md5 = BinaryUtil.calculateMd5Str(filePath)
                    Log.d("計算md5耗時", "${System.currentTimeMillis() - start}")
                    activity?.runOnUiThread{
                        result.success(md5)
                    }
                }
            }

            "upload" -> {
                selectFile(call.argument<String>("bucketName") as String)
            }

            "pause_upload" -> {
                val objectKey = call.argument<String>("objectKey") as String
                val pauseResult = ossService?.pauseUpload(objectKey)
                result.success(pauseResult)
            }

            "resume_upload" -> {
                val objectKey = call.argument<String>("objectKey") as String
                val resumeResult = ossService?.resumeUpload(objectKey)
                result.success(resumeResult)
            }

            else -> result.notImplemented()
        }
    }

    override fun onListen(arguments: Any?, events: EventSink?) {
        startListening(arguments, events)
    }

    override fun onCancel(arguments: Any?) {
        cancelListening(arguments)
    }

    private var listeners = HashMap<Any?, Runnable>()

    private fun startListening(listener: Any?, emitter: EventSink?) {
        val handler = Handler(Looper.getMainLooper())
        listeners[listener] = object : Runnable {
            override fun run() {
                if (listeners.containsKey(listener)) {
                    // Send some value to callback
                    emitter?.success("Hello listener! " + System.currentTimeMillis() / 1000)
                    handler.postDelayed(this, 1000)
                }
            }
        }
        // Run task
        handler.postDelayed(listeners[listener]!!, 1000)
    }

    private fun cancelListening(listener: Any?) {
        listeners.remove(listener);
    }

    /**
     * 初始化 OSS 客户端
     */
    private fun initOSSClient(params: HashMap<String, Any>) {
        Log.d("参数", "$params")
        val endpoint = params["endpoint"] as String
        val accessKeyId = params["accessKeyId"] as String
        val secretKeyId = params["secretKeyId"] as String
        val securityToken = params["securityToken"] as String?


        val conf = ClientConfiguration().apply {
            connectionTimeout = 15 * 1000 // 连接超时，默认15秒

            socketTimeout = 15 * 1000 // socket超时，默认15秒

            maxConcurrentRequest = 5 // 最大并发请求数，默认5个

            maxErrorRetry = 2 // 失败后最大重试次数，默认2次
        }


        val credential = if (securityToken == null) {
            OSSPlainTextAKSKCredentialProvider(accessKeyId, secretKeyId)
        } else {
            OSSStsTokenCredentialProvider(accessKeyId, secretKeyId, securityToken)
        }
        ossService = OSSService.getInstance(
            oss = OSSClient(activity!!, endpoint, credential, conf),
            recordDirectory = activity!!.filesDir.absolutePath.toString() + "/oatos/oss_record/",
            uploadCallback = UploadCallback(
                this::uploadInitCallback,
                this::uploadProgressCallback,
                this::uploadSuccessCallback,
                this::uploadFailCallback
            )
        )
    }


    private fun selectFile(bucketName: String) {
        this.bucketName = bucketName
        activity?.run {
            val intent = Intent(
                Intent.ACTION_GET_CONTENT
            )
            intent.type = "*/*"
            startActivityForResult(intent, resultSelectFile)
        }
    }

    /**
     * 上传文件，回传上传进度，处理分片/断点续传
     */
    private fun upload(bucketName: String, fileUri: Uri, filePath: String) {
        val extension =
            if (filePath.contains(".")) {
                filePath.substring(filePath.lastIndexOf("."))
            } else {
                filePath
            }

        thread {
            val start = System.currentTimeMillis()
            Log.d("计算md5", filePath)
            val md5 = BinaryUtil.calculateMd5Str(filePath)
            val objectKey = "$md5$extension"
            Log.d("計算md5耗時", "${System.currentTimeMillis() - start}")
            Log.d("上传名称objectKey", objectKey)

            uploadInitCallback(UploadInitData(filePath = filePath, objectKey = objectKey))

            ossService?.upload(
                bucketName,
                objectKey,
                fileUri
            )
        }
    }

    private fun runOnUiThread(callback: () -> Unit) {
        activity?.runOnUiThread {
            callback()
        }
    }

    private fun uploadInitCallback(initData: UploadInitData) {
        runOnUiThread {
            methodChannel.invokeMethod("upload_init", MapUtil.toMap(initData))
        }
    }

    private fun uploadProgressCallback(
        progress: UploadProgressData
    ) {
        runOnUiThread {
            methodChannel.invokeMethod(
                "upload_progress",
                MapUtil.toMap(progress)
            )
        }
    }

    private fun uploadSuccessCallback(successData: UploadSuccessData) {
        runOnUiThread {
            methodChannel.invokeMethod(
                "upload_success",
                MapUtil.toMap(successData)
            )
        }
    }

    private fun uploadFailCallback(failData: UploadFailData) {
        runOnUiThread {
            methodChannel.invokeMethod(
                "upload_fail",
                MapUtil.toMap(failData)
            )
        }
    }
}
