package com.wombat.aliyun_oss.service

import android.net.Uri
import android.util.Log
import com.alibaba.sdk.android.oss.ClientException
import com.alibaba.sdk.android.oss.OSS
import com.alibaba.sdk.android.oss.ServiceException
import com.alibaba.sdk.android.oss.callback.OSSCompletedCallback
import com.alibaba.sdk.android.oss.callback.OSSProgressCallback
import com.alibaba.sdk.android.oss.common.OSSLog
import com.alibaba.sdk.android.oss.model.*
import com.wombat.aliyun_oss.listener.UploadCallback
import com.wombat.aliyun_oss.model.UploadFailData
import com.wombat.aliyun_oss.model.UploadProgressData
import com.wombat.aliyun_oss.model.UploadSuccessData
import com.wombat.aliyun_oss.model.UploadTaskData
import java.io.File


class OSSService private constructor(
    private val oss: OSS,
    private val recordDirectory: String,
    private val uploadCallback: UploadCallback
) {

    private var uploadTasks = mutableSetOf<UploadTaskData>()

    companion object {
        @Volatile
        private var INSTANCE: OSSService? = null

        fun getInstance(
            oss: OSS,
            recordDirectory: String,
            uploadCallback: UploadCallback
        ): OSSService {
            if (INSTANCE == null) {
                synchronized(this) {
                    if (INSTANCE == null) {
                        INSTANCE = OSSService(oss, recordDirectory, uploadCallback)
                    }
                }
            }
            return INSTANCE!!
        }
    }

    fun upload(
        bucketName: String,
        objectKey: String,
        fileUri: Uri
    ) {
        val uploadStart = System.currentTimeMillis()
        OSSLog.logDebug("upload start")
        if (objectKey.isEmpty()) {
            Log.w("AsyncUploadRequest", "ObjectNull")
            return
        }
        // 构造上传请求
        OSSLog.logDebug("create ResumableUploadRequest ")

        val recordDir = File(recordDirectory)

        // 确保断点记录的保存文件夹已存在，如果不存在则新建断点记录的保存文件夹。
        if (!recordDir.exists()) {
            recordDir.mkdirs()
        }

        val rq = ResumableUploadRequest(bucketName, objectKey, fileUri, recordDirectory)
        rq.setDeleteUploadOnCancelling(false)

        rq.progressCallback =
            OSSProgressCallback<ResumableUploadRequest> { request, currentSize, totalSize ->
                val uploadProgress = UploadProgressData(
                    objectKey = objectKey,
                    uploadId = request.uploadId,
                    totalBytesSent = currentSize,
                    totalBytesExpectedToSend = totalSize
                )
                uploadCallback.progressCallback(uploadProgress)
            }

        val task = oss.asyncResumableUpload(
            rq,
            object :
                OSSCompletedCallback<ResumableUploadRequest, ResumableUploadResult> {
                override fun onSuccess(
                    request: ResumableUploadRequest,
                    result: ResumableUploadResult
                ) {
                    Log.d("MultipartUpload", "UploadSuccess")
                    Log.d("ETag", result.eTag)
                    val uploadEnd = System.currentTimeMillis()
                    OSSLog.logDebug("上传耗时: " + (uploadEnd - uploadStart) / 1000f)
                    Log.d("上传耗时", "${(uploadEnd - uploadStart) / 1000f}")

                    val success = UploadSuccessData(
                        objectKey = objectKey,
                        uploadId = request.uploadId,
                        result = result.location
                    )
                    uploadCallback.successCallback(success)

                    uploadTasks.filter {
                        it.objectKey == objectKey
                    }.forEach {
                        uploadTasks.remove(it)
                    }

                }

                override fun onFailure(
                    request: ResumableUploadRequest,
                    clientException: ClientException?,
                    serviceException: ServiceException?
                ) {
                    var info = ""
                    // 请求异常
                    if (clientException != null) {
                        // 本地异常如网络异常等
                        clientException.printStackTrace()
                        info = clientException.toString()
                    }
                    if (serviceException != null) {
                        // 服务异常
                        Log.e("ErrorCode", serviceException.errorCode)
                        Log.e("RequestId", serviceException.requestId)
                        Log.e("HostId", serviceException.hostId)
                        Log.e("RawMessage", serviceException.rawMessage)
                        info = serviceException.toString()
                    }
                    Log.e("错误", "$request")
                    uploadCallback.failCallback(
                        UploadFailData(
                            objectKey = objectKey,
                            uploadId = request.uploadId,
                            error = info
                        )
                    )
                }
            })

        uploadTasks.add(UploadTaskData(objectKey, rq.uploadId, bucketName, fileUri, task))

    }

    fun pauseUpload(
        objectKey: String
    ): String? {
        uploadTasks.firstOrNull {
            it.objectKey == objectKey
        }?.run {
            task.cancel()
            return objectKey
        }
        return null
    }

    fun resumeUpload(
        objectKey: String
    ): String? {
        uploadTasks.firstOrNull {
            it.objectKey == objectKey
        }?.run {
            uploadTasks.remove(this)
            upload(bucketName, objectKey, fileUri)
            return bucketName
        }

        return null
    }
}